#include "mlocpsoa.h"


static double objsign;
static fint NERROR = -1;
#define asl cur_ASL

char mlocpsoa_version[]="MLOCPSOA v1.1";

/* This struct member names must be in alphabetic order,
   for binary search */

keyword keywds[] = {
  KW("cognitial"   , mloc_opt_d, (Char*)&opt.mu,
     "Cognitial parameter"),
  KW("fweight"     , mloc_opt_d, (Char*)&opt.fweight,
     "Final velocity weight"),
  KW("halton"      , mloc_opt_i, (Char*)&opt.halton,
     "Halton (=1) or uniform (=0) initial population"),
  KW("iweight"     , mloc_opt_d, (Char*)&opt.iweight,
     "Initial velocity weight"),
  KW("maxiter"     , mloc_opt_i, (Char*)&opt.maxiter,
     "Maximum iterations allowed"),
  KW("maxvf"       , mloc_opt_d, (Char*)&opt.maxvfactor,
     "Factor for maximum allowed velocity"),
  KW("mlocal"      , mloc_opt_i, (Char*)&opt.mlocal,
     "Multilocal (=1) or global (=0) optimization"),
  KW("prog"        , mloc_opt_i, (Char*)&opt.prog,
     "Number of allowed iteration without progress"),
  KW("size"        , mloc_opt_i, (Char*)&opt.s,
     "Swarm size"),
  KW("social"      , mloc_opt_d, (char*)&opt.nu,
     "Social factor (gradient factor)"),
  KW("verb"      , mloc_opt_i, (char*)&opt.verb,
     "Verbosity (0=quiet,1=short info,2=all info)"),
};


struct Option_Info Oinfo = { "mlocpsoa", "MLOCPSOA", "mlocpsoa_options",
			     keywds, nkeywds, 1, mlocpsoa_version, 0, NULL};




/**********************************************
Set options. String type
**********************************************/
char *mloc_opt_s(Option_Info *oi, keyword *kw, char *value)
{
  char *s;


  /* never echo options */
  oi->option_echo &= ~ASL_OI_echo; 


  if(!strncmp("method", kw->name, 6)){
    s=value;
    while(*s!=' ' && *s!=0)
      s++;
    if(s<=value)
      return value;

    if(!strncmp("disc_hett", value, 9)){
      *(int *)kw->info=0; /*DISC_METHOD;*/
      printf("Discretization method selected Hettich version\n");
      return s;
    }

    /* unknown method */
    return value;
  }


  /* not implemented option */
  return value;
}


/**********************************************
Set options. Integer type
**********************************************/
char *mloc_opt_i(Option_Info *oi, keyword *kw, char *value)
{
  long optval;
  char *s;

  /* never echo options */
  oi->option_echo &= ~ASL_OI_echo;

  optval=strtol(value, &s, 10);
  if(s > value){
    /* existing integer number */
    *(int *)kw->info=(int)optval;
    if(opt.verb)
      printf("\nDefault option %s=%d accepted\n", kw->name, *(int *)kw->info);
    return s;
  }

return value;
}


/**********************************************
Set options. Double type
**********************************************/
char *mloc_opt_d(Option_Info *oi, keyword *kw, char *value)
{
  double optval;
  char *s;

  /* never echo options */
  oi->option_echo &= ~ASL_OI_echo;  

  optval=strtod(value, &s);

  if(s > value){
    /* existing double number */
    *(double *)kw->info=optval;
    if(opt.verb)
      printf("\nDefault option %s=%.6f accepted\n", kw->name, *(double *)kw->info);
    return s;
  }

return value;
}


static Jmp_buf Jb;

void catchfpe(int n)
{
  report_where(asl);
  printf("\nFloating point error.\n");
  fflush(stdout);
  longjmp(Jb.jb,1);
}


int main(int argc, char **argv)
{
  char *stub;
  ASL *asl;
  FILE *nl;

  fint m, n, no, nz, mxr, mxc;

  int i, exit_code, warnings=0;
  double *sol=NULL;
  double *f=NULL;
  double *lb, *ub, *lbt, *ubt, *tmp;
  
  
  asl = ASL_alloc(ASL_read_fg);

  stub = getstops(argv, &Oinfo);

  nl=jac_dim_ASL(asl, stub, &m, &n, &no, &nz, &mxr, &mxc, (fint)strlen(stub));
  if (!nl){
    printf("Error: Can't read problem\n");
    exit(1);
  }

  if(m){
    printf("\n\nWarning:\n");
    printf("Warning: Problem has %d constraint",m);
      if(m>1)printf("s\n");
      else printf("\n");
    printf("Warning: MLOCPSOA does not support constrained optimization problems\n", m);
    printf("Warning: Constraints will be ignored\n", m);
    printf("Warning:\n");
    warnings++;
  };


  want_deriv = opt.mlocal; /* no derivs in the global case */


  fg_read_ASL(asl, nl, 0);

  if(n_obj<1){
    printf("Error: At least one objective is requested\n");
    exit(1);
  }

  if(n_obj>1){
    printf("\nWarning: Current implentation only supports one objective function\n");
    printf("\nWarning: Considering only the first objective\n");
    warnings++;
  }

  dense_j();
  
  objsign = objtype[0] ? 1. : -1.;
  /* objsign = -1 minimization problem */
  /* objsign =  1 maximization problem */


  err_jmp1 = &Jb;
  if (!setjmp(Jb.jb)){
    signal(SIGFPE, catchfpe);

    /* lower and upper bounds on variables */
    lbt=lb=malloc(n*sizeof(double));    
    ubt=ub=malloc(n*sizeof(double));
    if(!lb || !ub){
      printf("Error: Unable to allocate memory for variable bounds\n");
      exit(1);
    }
    tmp=LUv;
    for(i=0;i<n;i++){
      if(*tmp<-Inf){
	printf("\nWarning: Lower bound on variable %d not set\n",i);
	warnings++;
      }
      *lbt++=*tmp++;
      if(*tmp>Inf){
	printf("\nWarning: Upper bound on variable %d not set\n",i);
	warnings++;
      }
      *ubt++=*tmp++;
    }

    exit_code=mlocpsoa(n, &objfun, lb, ub, &sol, f);

    write_sol("Solution found by MLOCPSOA", sol, NULL, &Oinfo);
  }

  /* free allocated memory */
  free(lb);
  free(ub);
  free(sol);

  if(warnings){
    if(warnings==1)printf("\nWarning: There was 1 warning\n");
    else printf("\nWarning: There were %d warnings\n", warnings);
  }


  if(opt.verb)printf("\n%s\n", exit_codes[exit_code].msg);

  return exit_code;
}

/*
Function to be passed to the mlocpsoa function.
args in:
      n -> dimension
      x -> point
      mode -> 0: only objective
              1: only gradient
	      2: objective and gradient
args out:
      grd -> gradient of objective at x
*/
double objfun(int n, double *x, int mode, double *grd)
{
  double *g;


  if(mode && grd){
    objgrd(0, x, grd, NULL);
    if(objsign < 0)
      for(g=grd+n-1;g>=grd;g--)
	*g=-*g;
  }

  if(mode!=1)
    return objsign*objval(0,x,NULL);
  else
    return 0.0;
}
