###############################################################################
# Objective: Generic
# Constraints: Simple bounds
###############################################################################
#  Randomly generated test problem
#   As described by A. Ismael F. Vaz and Edite M.G.P. Fernandes
#	"On randomly generated semi-infinite programming test problems",
#	 in Technical Report ALG/EF/2-2002
#	
#   Coded by A. Ismael F. Vaz 26/08/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
#
#  Adapted for multilocal optimization by A. Ismael F. Vaz 23/04/04
#
#  This is a regular (finite) maximization problems ... Not random any more
#
###############################################################################


param p   := 6;               # Number of infinite variables
param L   := 7;               # Constant used in uniform random generation 
param Lb  := 5;               # Constant used in uniform random generation 
param La  := 4;               # Constant used in uniform random generation 
param pi  := atan(1)*4;       # Pi for the sine and cosine functions


var t{i in 1..p};

param c_eta := (L/2)*(0.234211);            # Data for the signomials
param a{1..4} := La*(0.345938-0.5);      # Data for the signomials
param b_l{1..p};       # Data for the extended signomials

let b_l[1] := Lb*(0.489057); # 2.445285
let b_l[2] := Lb*(0.349347); # 1.746735
let b_l[3] := Lb*(0.278919); # 1.394595
let b_l[4] := Lb*(0.237853); # 1.189265
let b_l[5] := Lb*(0.891343); # 4.456715
let b_l[6] := Lb*(0.097895); # .489475


param s_u := prod {i in 1..p} (floor(b_l[i]+0.5));


var ext_sig = (c_eta*(prod {zeta in 1..4}(a[zeta])))
			*(prod {l in 1..p} (sin(t[l]*b_l[l]*pi)^2));


###############################################################################
# Define problem
###############################################################################

maximize ft:
	ext_sig;

subject to tbound {i in 1..p}:
	0 <= t[i] <= 1;


option solver mlocpsoa;
solve;


###############################################################################
# Solution found
###############################################################################
display t;
display ft;